@@ -1,5 +1,6 @@
 #
 # Copyright (c) 2013 Matwey V. Kornilov <matwey.kornilov@gmail.com>
+# Copyright (c) 2013 Fredrik Ahlberg <fredrik@z80.se>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -21,11 +22,25 @@
 #
 
 from construct import *
-from eetf_construct import term
+from pybeam.eetf_construct import term, external_term
+import codecs
 
-chunk_atom = Struct("chunk_atom",
-	UBInt32("len"),
-	Array(lambda ctx: ctx.len, PascalString("atom"))
+erl_version_magic = Magic(b'\x83')
+
+chunk_atom = Rename("chunk_atom", PrefixedArray(PascalString("atom", encoding="latin1"), length_field = UBInt32("len")))
+
+chunk_attr = Rename("chunk_attr", external_term)
+
+chunk_cinf = Rename("chunk_cinf", external_term)
+
+chunk_code = Struct("chunk_code",
+	UBInt32("headerlen"),
+	UBInt32("set"),
+	UBInt32("opcode_max"),
+	UBInt32("labels"),
+	UBInt32("functions"),
+	Bytes("skip", lambda ctx: ctx.headerlen-16),
+	Bytes("code", lambda ctx: ctx._.size-ctx.headerlen-4),
 	)
 
 chunk_expt = Struct("chunk_expt",
@@ -48,6 +63,25 @@ chunk_impt = Struct("chunk_impt",
 	)
 	)
 
+chunk_litt = Struct("chunk_litt",
+	UBInt32("len_uncompressed"),
+	TunnelAdapter(
+		ExprAdapter(Bytes("data", length = lambda ctx: ctx._.size-4),
+			encoder = lambda obj,ctx: codecs.encode(obj,"zlib_codec"),
+	                decoder = lambda obj,ctx: codecs.decode(obj,"zlib_codec")
+		),
+		Struct("uncompressed",
+			UBInt32("len"),
+			Array(lambda ctx: ctx.len, Struct("entry",
+					UBInt32("len"),
+					erl_version_magic,
+					term
+				)
+			)
+		)
+	)
+	)
+
 chunk_loct = Struct("chunk_loct",
 	UBInt32("len"),
 	Array(lambda ctx: ctx.len, Struct("entry",
@@ -58,44 +92,45 @@ chunk_loct = Struct("chunk_loct",
 	)
 	)
 
-chunk_attr = Struct("chunk_attr",
-	Const(UBInt8("131"),131),
-	term
-	)
-
-chunk_cinf = Struct("chunk_cinf",
-	Const(UBInt8("131"),131),
-	term
+chunk_strt = Struct("chunk_strt",
+	PascalString("string", length_field = UBInt32("len"), encoding="latin1")
 	)
 
 chunk = Struct("chunk",
 	String("chunk_name",4),
 	UBInt32("size"),
-	SeqOfOne("payload",
-		Switch("payload", lambda ctx: ctx.chunk_name,
-			{
-			"Atom" : chunk_atom,
-			"ExpT" : chunk_expt,
-			"ImpT" : chunk_impt,
-#			"Code" : chunk_code,
-#			"StrT" : chunk_strt,
-			"Attr" : chunk_attr,
-			"CInf" : chunk_cinf,
-			"LocT" : chunk_loct,
-#			"Trac" : chunk_trac,
-			},
-			default = Bytes("skip", lambda ctx: ctx.size)
+	If(lambda ctx: ctx.size > 0,
+		SeqOfOne("payload",
+			Switch("payload", lambda ctx: ctx.chunk_name,
+				{
+#				"Abst" : chunk_abst,
+				b"Atom" : chunk_atom,
+				b"Attr" : chunk_attr,
+				b"CInf" : chunk_cinf,
+				b"Code" : chunk_code,
+				b"ExpT" : chunk_expt,
+#				"FunT" : chunk_funt,
+				b"ImpT" : chunk_impt,
+#				"Line" : chink_line,
+				b"LitT" : chunk_litt,
+				b"LocT" : chunk_loct,
+#				"StrT" : chunk_strt,
+#				"Trac" : chunk_trac,
+				},
+				default = Bytes("skip", lambda ctx: ctx.size)
+			),
+			Padding(lambda ctx: (4 - ctx.size % 4) % 4, pattern = "\00"),
+			nested = False,
 		),
-		Padding(lambda ctx: (4 - ctx.size % 4) % 4, pattern = "\00"),
-		nested = False,
-	)
+		[]
+		)
 	)
 
 beam = Struct("beam",
-	Const(String('for1',4),'FOR1'),
+	Const(String('for1',4),b'FOR1'),
 	UBInt32("size"),
-	Const(String('beam',4),'BEAM'),
-	GreedyRange(chunk),
+	Const(String('beam',4),b'BEAM'),
+	OptionalGreedyRange(chunk),
 	Terminator,
 	)
 
